import { Alert, Box, Button, Container, Paper, Step, StepLabel, Stepper, TextField, Typography } from '@mui/material';
import React, { useState } from 'react';
import { useNavigate } from 'react-router-dom';
import { useAuth } from '../../contexts/AuthContext';

const steps = ['Phone Verification', 'Email', 'Set Passcode'];

export const RegisterForm = () => {
  const { register, verifyOtp, setEmail, setPasscode } = useAuth();
  const navigate = useNavigate();
  const [activeStep, setActiveStep] = useState(0);
  const [loading, setLoading] = useState(false);
  const [error, setError] = useState('');
  const [formData, setFormData] = useState({
    countryCode: '+962',
    phoneNumber: '',
    otp: '',
    email: '',
    passcode: '',
    confirmPasscode: '',
    otpRequested: false
  });

  const handleSubmit = async (e: React.FormEvent) => {
    e.preventDefault();
    setError('');
    setLoading(true);

    try {
      switch (activeStep) {
        case 0:
          if (!formData.otpRequested) {
            // Request OTP
            await register(formData.countryCode, formData.phoneNumber);
            setFormData(prev => ({ ...prev, otpRequested: true }));
          } else {
            // Verify OTP
            await verifyOtp(formData.countryCode, formData.phoneNumber, formData.otp);
            setActiveStep(1);
          }
          break;
        case 1:
          await setEmail(formData.email);
          setActiveStep(2);
          break;
        case 2:
          if (formData.passcode !== formData.confirmPasscode) {
            throw new Error('Passcodes do not match');
          }
          await setPasscode(formData.passcode);
          navigate('/dashboard');
          break;
      }
    } catch (err) {
      setError(err instanceof Error ? err.message : 'Registration failed');
    } finally {
      setLoading(false);
    }
  };

  const handleInputChange = (e: React.ChangeEvent<HTMLInputElement>) => {
    const { name, value } = e.target;
    setFormData((prev) => ({
      ...prev,
      [name]: value,
    }));
  };

  const renderStepContent = () => {
    switch (activeStep) {
      case 0:
        return (
          <>
            <TextField
              fullWidth
              margin="normal"
              label="Phone Number"
              name="phoneNumber"
              value={formData.phoneNumber}
              onChange={handleInputChange}
              placeholder="7XXXXXXXX"
              required
              disabled={formData.otpRequested}
              sx={{
                '& .MuiOutlinedInput-root': {
                  borderRadius: 2,
                }
              }}
            />
            {formData.otpRequested && (
              <TextField
                fullWidth
                margin="normal"
                label="OTP"
                name="otp"
                value={formData.otp}
                onChange={handleInputChange}
                placeholder="Enter OTP"
                required
                autoFocus
                sx={{
                  '& .MuiOutlinedInput-root': {
                    borderRadius: 2,
                  }
                }}
              />
            )}
          </>
        );
      case 1:
        return (
          <TextField
            fullWidth
            margin="normal"
            label="Email"
            name="email"
            type="email"
            value={formData.email}
            onChange={handleInputChange}
            required
            autoFocus
            sx={{
              '& .MuiOutlinedInput-root': {
                borderRadius: 2,
              }
            }}
          />
        );
      case 2:
        return (
          <>
            <TextField
              fullWidth
              margin="normal"
              label="Passcode"
              name="passcode"
              type="password"
              value={formData.passcode}
              onChange={handleInputChange}
              required
              autoFocus
              sx={{
                '& .MuiOutlinedInput-root': {
                  borderRadius: 2,
                }
              }}
            />
            <TextField
              fullWidth
              margin="normal"
              label="Confirm Passcode"
              name="confirmPasscode"
              type="password"
              value={formData.confirmPasscode}
              onChange={handleInputChange}
              required
              sx={{
                '& .MuiOutlinedInput-root': {
                  borderRadius: 2,
                }
              }}
            />
          </>
        );
      default:
        return null;
    }
  };

  return (
    <Box
      sx={{
        minHeight: '100vh',
        display: 'flex',
        flexDirection: 'column',
        backgroundColor: 'background.default',
      }}
    >
      <Container maxWidth="sm" sx={{ flex: 1, display: 'flex', flexDirection: 'column', justifyContent: 'center' }}>
        <Box sx={{ mb: 4, textAlign: 'center' }}>
          <Typography variant="h3" component="h1" gutterBottom sx={{ fontWeight: 700, color: 'primary.main' }}>
            Zod Alkhair | API TEST
          </Typography>
          <Typography variant="h6" sx={{ color: 'text.secondary', mb: 4 }}>
            signup
          </Typography>
        </Box>

        <Paper
          elevation={0}
          sx={{
            p: 4,
            borderRadius: 3,
            border: '1px solid',
            borderColor: 'divider',
            backgroundColor: 'background.paper'
          }}
        >
          <Stepper activeStep={activeStep} sx={{ mb: 4 }}>
            {steps.map((label) => (
              <Step key={label}>
                <StepLabel>{label}</StepLabel>
              </Step>
            ))}
          </Stepper>

          {error && (
            <Alert severity="error" sx={{ mb: 3 }}>
              {error}
            </Alert>
          )}

          <Box component="form" onSubmit={handleSubmit}>
            {renderStepContent()}

            <Button
              type="submit"
              fullWidth
              variant="contained"
              size="large"
              sx={{
                mt: 3,
                mb: 2,
                height: 48,
                borderRadius: 2,
                textTransform: 'none',
                fontSize: '1rem'
              }}
              disabled={loading}
            >
              {loading ? 'Processing...' : activeStep === 0 && !formData.otpRequested ? 'Send OTP' : 'Continue'}
            </Button>

            <Button
              fullWidth
              variant="text"
              sx={{
                textTransform: 'none',
                fontSize: '1rem',
                color: 'text.secondary',
                '&:hover': {
                  color: 'primary.main'
                }
              }}
              onClick={() => navigate('/login')}
            >
              sign in
            </Button>
          </Box>
        </Paper>
      </Container>
    </Box>
  );
};
