import {
  Alert,
  Box,
  Button,
  Checkbox,
  FormControl,
  FormControlLabel,
  Grid,
  InputLabel,
  MenuItem,
  Paper,
  Select,
  SelectChangeEvent,
  TextField,
  Typography,
} from '@mui/material';
import { AxiosError } from 'axios';
import React, { useEffect, useState } from 'react';
import { useNavigate } from 'react-router-dom';
import { juniorsApi, tasksApi } from '../../api/client';
import { ApiError } from '../../types/api';
import { DocumentType } from '../../types/document';
import { Junior, PaginatedResponse } from '../../types/junior';
import { CreateTaskRequest } from '../../types/task';
import { DocumentUpload } from '../document/DocumentUpload';

export const AddTaskForm = () => {
  const navigate = useNavigate();
  const [loading, setLoading] = useState(false);
  const [error, setError] = useState('');
  const [formData, setFormData] = useState<CreateTaskRequest>({
    title: '',
    description: '',
    dueDate: '',
    rewardAmount: 0,
    isProofRequired: false,
    juniorId: '',
    imageId: '',
  });

  const [juniors, setJuniors] = useState<Junior[]>([]);

  const handleSubmit = async (e: React.FormEvent) => {
    e.preventDefault();
    console.log('Form data:', formData);

    setError('');
    setLoading(true);

    try {
      if (!formData.imageId) {
        console.log('Proof is required but no image uploaded');
      }

      console.log('Submitting data:', formData);
      const dataToSubmit = {
        ...formData,
        rewardAmount: Number(formData.rewardAmount),
        imageId: formData.imageId,
      };
      await tasksApi.createTask(dataToSubmit);
      navigate('/tasks');
    } catch (err) {
      console.error('Create junior error:', err);
      if (err instanceof AxiosError && err.response?.data) {
        const apiError = err.response.data as ApiError;
        const messages = Array.isArray(apiError.message)
          ? apiError.message.map((m) => `${m.field}: ${m.message}`).join('\n')
          : apiError.message;
        setError(messages);
      } else {
        setError(err instanceof Error ? err.message : 'Failed to create Task');
      }
    } finally {
      setLoading(false);
    }
  };

  const handleInputChange = (e: React.ChangeEvent<HTMLInputElement>) => {
    const { name, value } = e.target;
    console.log(name, value);
    setFormData((prev) => ({
      ...prev,
      [name]: value,
    }));
  };

  const fetchJuniors = async () => {
    try {
      const response = await juniorsApi.getJuniors(1, 50);
      const data = response.data as PaginatedResponse<Junior>;
      setJuniors(data.data);
    } catch (err) {
      console.error('Failed to load juniors:', err);
    }
  };

  const handleSelectChange = (e: SelectChangeEvent) => {
    const { name, value } = e.target;
    setFormData((prev) => ({
      ...prev,
      [name as string]: value,
    }));
  };

  useEffect(() => {
    console.log('Form data updated:', formData);
  }, [formData]);

  useEffect(() => {
    fetchJuniors();
  }, []);

  const handleTaskImageUpload = (documentId: string) => {
    console.log('task image ID uploaded:', documentId);
    setFormData((prev) => ({
      ...prev,
      imageId: documentId,
    }));
  };

  const handleCheckedInputChange = (e: React.ChangeEvent<HTMLInputElement>) => {
    setFormData((prev) => ({
      ...prev,
      isProofRequired: e.target.checked,
    }));
  };

  return (
    <Box p={3}>
      <Typography variant="h4" gutterBottom>
        Add New Task
      </Typography>

      <Paper sx={{ p: 3, maxWidth: 600, mx: 'auto' }}>
        {error && (
          <Alert severity="error" sx={{ mb: 3, whiteSpace: 'pre-line' }}>
            {error}
          </Alert>
        )}

        <Box component="form" onSubmit={handleSubmit}>
          <Grid container spacing={3}>
            <Grid item xs={12} sm={12}>
              <TextField
                fullWidth
                label="Title"
                name="title"
                value={formData.title}
                onChange={handleInputChange}
                placeholder="Task Title"
                required
              />
            </Grid>
            <Grid item xs={12} sm={12}>
              <TextField
                fullWidth
                label="Description"
                name="description"
                value={formData.description}
                onChange={handleInputChange}
                placeholder="Task Description"
                required
              />
            </Grid>

            <Grid item xs={12} sm={12}>
              <TextField
                fullWidth
                label="Due Date"
                name="dueDate"
                type="date"
                value={formData.dueDate}
                onChange={handleInputChange}
                required
                InputLabelProps={{
                  shrink: true,
                }}
              />
            </Grid>

            <Grid item xs={12} sm={12}>
              <TextField
                fullWidth
                label="Reward Amount"
                name="rewardAmount"
                type="number"
                value={formData.rewardAmount}
                onChange={handleInputChange}
                required
              />
            </Grid>

            <Grid item xs={12}>
              <FormControl fullWidth>
                <InputLabel>Junior</InputLabel>
                <Select name="juniorId" value={formData.juniorId} label="Junior" onChange={handleSelectChange}>
                  <MenuItem value="">Select Junior</MenuItem>
                  {juniors.map((junior) => (
                    <MenuItem key={junior.id} value={junior.id}>
                      {junior.fullName}
                    </MenuItem>
                  ))}
                </Select>
              </FormControl>
            </Grid>

            <Grid item xs={12} sm={12}>
              <DocumentUpload
                documentType={DocumentType.PASSPORT}
                label="Upload Task Image"
                onUploadSuccess={handleTaskImageUpload}
              />
              {formData.imageId && (
                <Typography variant="caption" color="success.main" sx={{ mt: 1, display: 'block' }}>
                  Task Image uploaded (ID: {formData.imageId})
                </Typography>
              )}
            </Grid>

            <Grid item xs={12}>
              <FormControl fullWidth>
                <FormControlLabel
                  control={
                    <Checkbox checked={formData.isProofRequired} onChange={handleCheckedInputChange} color="primary" />
                  }
                  label="Proof Required"
                />
              </FormControl>
            </Grid>
          </Grid>

          <Box sx={{ mt: 3, display: 'flex', gap: 2, justifyContent: 'flex-end' }}>
            <Button variant="outlined" onClick={() => navigate('/juniors')}>
              Cancel
            </Button>
            <Button type="submit" variant="contained" disabled={loading}>
              {loading ? 'Adding...' : 'Add Task'}
            </Button>
          </Box>
        </Box>
      </Paper>
    </Box>
  );
};
