import { Body, Controller, Get, HttpCode, HttpStatus, Param, Patch, Post, Query, UseGuards } from '@nestjs/common';
import { ApiBearerAuth, ApiTags } from '@nestjs/swagger';
import { Roles } from '~/auth/enums';
import { IJwtPayload } from '~/auth/interfaces';
import { AllowedRoles, AuthenticatedUser } from '~/common/decorators';
import { RolesGuard } from '~/common/guards';
import { ApiDataPageResponse, ApiDataResponse, ApiLangRequestHeader } from '~/core/decorators';
import { PageOptionsRequestDto } from '~/core/dtos';
import { CustomParseUUIDPipe } from '~/core/pipes';
import { ResponseFactory } from '~/core/utils';
import { CreateAllowanceChangeRequestDto } from '../dtos/request';
import { AllowanceChangeRequestResponseDto } from '../dtos/response';
import { AllowanceChangeRequestsService } from '../services';

@Controller('allowance-change-requests')
@ApiTags('Allowance Change Requests')
@ApiBearerAuth()
@ApiLangRequestHeader()
export class AllowanceChangeRequestController {
  constructor(private readonly allowanceChangeRequestsService: AllowanceChangeRequestsService) {}

  @Post()
  @UseGuards(RolesGuard)
  @AllowedRoles(Roles.JUNIOR)
  @HttpCode(HttpStatus.NO_CONTENT)
  requestAllowanceChange(@AuthenticatedUser() { sub }: IJwtPayload, @Body() body: CreateAllowanceChangeRequestDto) {
    return this.allowanceChangeRequestsService.createAllowanceChangeRequest(sub, body);
  }

  @Get()
  @UseGuards(RolesGuard)
  @AllowedRoles(Roles.GUARDIAN)
  @ApiDataPageResponse(AllowanceChangeRequestResponseDto)
  async findAllowanceChangeRequests(@AuthenticatedUser() { sub }: IJwtPayload, @Query() query: PageOptionsRequestDto) {
    const [requests, itemCount] = await this.allowanceChangeRequestsService.findAllowanceChangeRequests(sub, query);

    return ResponseFactory.dataPage(
      requests.map((request) => new AllowanceChangeRequestResponseDto(request)),
      {
        itemCount,
        page: query.page,
        size: query.size,
      },
    );
  }

  @Get('/:changeRequestId')
  @UseGuards(RolesGuard)
  @AllowedRoles(Roles.GUARDIAN)
  @ApiDataResponse(AllowanceChangeRequestResponseDto)
  async findAllowanceChangeRequestById(
    @AuthenticatedUser() { sub }: IJwtPayload,
    @Param('changeRequestId', CustomParseUUIDPipe) changeRequestId: string,
  ) {
    const request = await this.allowanceChangeRequestsService.findAllowanceChangeRequestById(sub, changeRequestId);

    return ResponseFactory.data(new AllowanceChangeRequestResponseDto(request));
  }

  @Patch(':changeRequestId/approve')
  @UseGuards(RolesGuard)
  @AllowedRoles(Roles.GUARDIAN)
  @HttpCode(HttpStatus.NO_CONTENT)
  approveAllowanceChangeRequest(
    @AuthenticatedUser() { sub }: IJwtPayload,
    @Param('changeRequestId', CustomParseUUIDPipe) changeRequestId: string,
  ) {
    return this.allowanceChangeRequestsService.approveAllowanceChangeRequest(sub, changeRequestId);
  }

  @Patch(':changeRequestId/reject')
  @UseGuards(RolesGuard)
  @AllowedRoles(Roles.GUARDIAN)
  @HttpCode(HttpStatus.NO_CONTENT)
  rejectAllowanceChangeRequest(
    @AuthenticatedUser() { sub }: IJwtPayload,
    @Param('changeRequestId', CustomParseUUIDPipe) changeRequestId: string,
  ) {
    return this.allowanceChangeRequestsService.rejectAllowanceChangeRequest(sub, changeRequestId);
  }
}
