import { BadRequestException, Injectable, Logger } from '@nestjs/common';
import { FindOptionsWhere } from 'typeorm';
import { PageOptionsRequestDto } from '~/core/dtos';
import { OciService } from '~/document/services';
import { CreateAllowanceChangeRequestDto } from '../dtos/request';
import { AllowanceChangeRequest } from '../entities';
import { AllowanceChangeRequestStatus } from '../enums';
import { AllowanceChangeRequestsRepository } from '../repositories';
import { AllowancesService } from './allowances.service';

@Injectable()
export class AllowanceChangeRequestsService {
  private readonly logger = new Logger(AllowanceChangeRequestsService.name);
  constructor(
    private readonly allowanceChangeRequestsRepository: AllowanceChangeRequestsRepository,
    private readonly ociService: OciService,
    private readonly allowanceService: AllowancesService,
  ) {}

  async createAllowanceChangeRequest(juniorId: string, body: CreateAllowanceChangeRequestDto) {
    this.logger.log(`Creating allowance change request for junior ${juniorId}`);
    const allowance = await this.allowanceService.validateAllowanceForJunior(juniorId, body.allowanceId);

    if (allowance.amount === body.amount) {
      this.logger.error(`Amount is the same as the current allowance amount`);
      throw new BadRequestException('ALLOWANCE_CHANGE_REQUEST.SAME_AMOUNT');
    }

    const requestWithTheSameAmount = await this.findAllowanceChangeRequestBy({
      allowanceId: body.allowanceId,
      amount: body.amount,
      status: AllowanceChangeRequestStatus.PENDING,
    });

    if (requestWithTheSameAmount) {
      this.logger.error(`There is a pending request with the same amount`);
      throw new BadRequestException('ALLOWANCE_CHANGE_REQUEST.SAME_AMOUNT_PENDING');
    }

    return this.allowanceChangeRequestsRepository.createAllowanceChangeRequest(body.allowanceId, body);
  }

  findAllowanceChangeRequestBy(where: FindOptionsWhere<AllowanceChangeRequest>) {
    this.logger.log(`Finding allowance change request by ${JSON.stringify(where)}`);
    return this.allowanceChangeRequestsRepository.findAllowanceChangeRequestBy(where);
  }

  async approveAllowanceChangeRequest(guardianId: string, requestId: string) {
    this.logger.log(`Approving allowance change request ${requestId} by guardian ${guardianId}`);
    const request = await this.findAllowanceChangeRequestBy({ id: requestId, allowance: { guardianId } });

    if (!request) {
      this.logger.error(`Allowance change request ${requestId} not found for guardian ${guardianId}`);
      throw new BadRequestException('ALLOWANCE_CHANGE_REQUEST.NOT_FOUND');
    }
    if (request.status === AllowanceChangeRequestStatus.APPROVED) {
      this.logger.error(`Allowance change request ${requestId} already approved`);
      throw new BadRequestException('ALLOWANCE_CHANGE_REQUEST.ALREADY_APPROVED');
    }
    return this.allowanceChangeRequestsRepository.updateAllowanceChangeRequestStatus(
      requestId,
      AllowanceChangeRequestStatus.APPROVED,
    );
  }

  async rejectAllowanceChangeRequest(guardianId: string, requestId: string) {
    this.logger.log(`Rejecting allowance change request ${requestId} by guardian ${guardianId}`);
    const request = await this.findAllowanceChangeRequestBy({ id: requestId, allowance: { guardianId } });

    if (!request) {
      this.logger.error(`Allowance change request ${requestId} not found for guardian ${guardianId}`);
      throw new BadRequestException('ALLOWANCE_CHANGE_REQUEST.NOT_FOUND');
    }
    if (request.status === AllowanceChangeRequestStatus.REJECTED) {
      this.logger.error(`Allowance change request ${requestId} already rejected`);
      throw new BadRequestException('ALLOWANCE_CHANGE_REQUEST.ALREADY_REJECTED');
    }
    return this.allowanceChangeRequestsRepository.updateAllowanceChangeRequestStatus(
      requestId,
      AllowanceChangeRequestStatus.REJECTED,
    );
  }

  async findAllowanceChangeRequests(
    guardianId: string,
    query: PageOptionsRequestDto,
  ): Promise<[AllowanceChangeRequest[], number]> {
    this.logger.log(`Finding allowance change requests for guardian ${guardianId}`);
    const [requests, itemCount] = await this.allowanceChangeRequestsRepository.findAllowanceChangeRequests(
      guardianId,
      query,
    );

    await this.prepareAllowanceChangeRequestsImages(requests);

    this.logger.log(`Returning allowance change requests for guardian ${guardianId}`);
    return [requests, itemCount];
  }

  async findAllowanceChangeRequestById(guardianId: string, requestId: string) {
    this.logger.log(`Finding allowance change request ${requestId} for guardian ${guardianId}`);
    const request = await this.allowanceChangeRequestsRepository.findAllowanceChangeRequestBy(
      {
        id: requestId,
        allowance: { guardianId },
      },
      true,
    );

    if (!request) {
      this.logger.error(`Allowance change request ${requestId} not found for guardian ${guardianId}`);
      throw new BadRequestException('ALLOWANCE_CHANGE_REQUEST.NOT_FOUND');
    }

    await this.prepareAllowanceChangeRequestsImages([request]);

    this.logger.log(`Allowance change request ${requestId} found successfully`);
    return request;
  }

  private prepareAllowanceChangeRequestsImages(requests: AllowanceChangeRequest[]) {
    this.logger.log(`Preparing allowance change requests images`);
    return Promise.all(
      requests.map(async (request) => {
        const profilePicture = request.allowance.junior.customer.profilePicture;
        if (profilePicture) {
          profilePicture.url = await this.ociService.generatePreSignedUrl(profilePicture);
        }
      }),
    );
  }
}
