import { HttpService } from '@nestjs/axios';
import { Injectable, Logger, UnauthorizedException } from '@nestjs/common';
import { ConfigService } from '@nestjs/config';
import { JwtService } from '@nestjs/jwt';
import { OAuth2Client } from 'google-auth-library';
import jwkToPem from 'jwk-to-pem';
import { lastValueFrom } from 'rxjs';
import { ApplePayload } from '../interfaces';

@Injectable()
export class Oauth2Service {
  private readonly logger = new Logger(Oauth2Service.name);
  private appleKeysEndpoint = 'https://appleid.apple.com/auth/keys';
  private appleIssuer = 'https://appleid.apple.com';
  private readonly googleWebClientId = this.configService.getOrThrow('GOOGLE_WEB_CLIENT_ID');
  private readonly googleAndroidClientId = this.configService.getOrThrow('GOOGLE_ANDROID_CLIENT_ID');
  private readonly googleIosClientId = this.configService.getOrThrow('GOOGLE_IOS_CLIENT_ID');
  private readonly client = new OAuth2Client();
  constructor(
    private readonly httpService: HttpService,
    private readonly jwtService: JwtService,
    private readonly configService: ConfigService,
  ) {}

  async verifyAppleToken(appleToken: string): Promise<ApplePayload> {
    try {
      const response = await lastValueFrom(this.httpService.get(this.appleKeysEndpoint));

      const keys = response.data.keys;

      const decodedHeader = this.jwtService.decode(appleToken, { complete: true })?.header;

      if (!decodedHeader) {
        this.logger.error(`Invalid apple token`);
        throw new UnauthorizedException();
      }

      const keyId = decodedHeader.kid;

      const appleKey = keys.find((key: any) => key.kid === keyId);

      if (!appleKey) {
        this.logger.error(`Invalid apple token`);
        throw new UnauthorizedException();
      }

      const publicKey = jwkToPem(appleKey);

      const payload = this.jwtService.verify(appleToken, {
        publicKey,
        algorithms: ['RS256'],
        audience: this.configService.getOrThrow('APPLE_CLIENT_ID'),
        issuer: this.appleIssuer,
      });

      return payload;
    } catch (error) {
      this.logger.error(`Error verifying apple token: ${error} `);
      throw new UnauthorizedException(error);
    }
  }

  async verifyGoogleToken(googleToken: string): Promise<any> {
    try {
      const ticket = await this.client.verifyIdToken({
        idToken: googleToken,
        audience: [this.googleWebClientId, this.googleAndroidClientId, this.googleIosClientId],
      });

      const payload = ticket.getPayload();

      if (!payload) {
        this.logger.error(`payload not found in google token`);
        throw new UnauthorizedException();
      }

      return payload;
    } catch (error) {
      this.logger.error(`Invalid google token`, error);
      throw new UnauthorizedException();
    }
  }
}
