import { MailerService } from '@nestjs-modules/mailer';
import { Injectable, Logger } from '@nestjs/common';
import { EventEmitter2, OnEvent } from '@nestjs/event-emitter';
import { PageOptionsRequestDto } from '~/core/dtos';
import { DeviceService } from '~/user/services';
import { OTP_BODY, OTP_TITLE } from '../../otp/constants';
import { OtpType } from '../../otp/enums';
import { ISendOtp } from '../../otp/interfaces';
import { SendEmailRequestDto } from '../dtos/request';
import { Notification } from '../entities';
import { EventType, NotificationChannel, NotificationScope } from '../enums';
import { NotificationsRepository } from '../repositories';
import { FirebaseService } from './firebase.service';
import { TwilioService } from './twilio.service';

@Injectable()
export class NotificationsService {
  private readonly logger = new Logger(NotificationsService.name);
  constructor(
    private readonly firebaseService: FirebaseService,
    private readonly notificationRepository: NotificationsRepository,
    private readonly twilioService: TwilioService,
    private readonly eventEmitter: EventEmitter2,
    private readonly deviceService: DeviceService,
    private readonly mailerService: MailerService,
  ) {}

  async getNotifications(userId: string, pageOptionsDto: PageOptionsRequestDto) {
    this.logger.log(`Getting notifications for user ${userId}`);
    const [[notifications, count], unreadCount] = await Promise.all([
      this.notificationRepository.getNotifications(userId, pageOptionsDto),
      this.notificationRepository.getUnreadNotificationsCount(userId),
    ]);

    this.logger.log(`Returning notifications for user ${userId}`);

    return { notifications, count, unreadCount };
  }

  createNotification(notification: Partial<Notification>) {
    this.logger.log(`Creating notification for user ${notification.userId}`);
    return this.notificationRepository.createNotification(notification);
  }

  markAsRead(userId: string) {
    this.logger.log(`Marking notifications as read for user ${userId}`);
    return this.notificationRepository.markAsRead(userId);
  }

  async sendEmailAsync(data: SendEmailRequestDto) {
    this.logger.log(`emitting ${EventType.NOTIFICATION_CREATED} event`);
    const notification = await this.createNotification({
      recipient: data.to,
      title: data.subject,
      message: '',
      scope: NotificationScope.USER_INVITED,
      channel: NotificationChannel.EMAIL,
    });
    return this.eventEmitter.emit(EventType.NOTIFICATION_CREATED, notification, data.data);
  }

  async sendOtpNotification(sendOtpRequest: ISendOtp, otp: string) {
    this.logger.log(`Sending OTP to ${sendOtpRequest.recipient}`);
    const notification = await this.createNotification({
      recipient: sendOtpRequest.recipient,
      title: OTP_TITLE,
      message: OTP_BODY.replace('{otp}', otp),
      scope: NotificationScope.OTP,
      channel: sendOtpRequest.otpType === OtpType.EMAIL ? NotificationChannel.EMAIL : NotificationChannel.SMS,
    });

    this.logger.log(`emitting ${EventType.NOTIFICATION_CREATED} event`);

    return this.eventEmitter.emit(EventType.NOTIFICATION_CREATED, notification);
  }

  private async sendPushNotification(userId: string, title: string, body: string) {
    this.logger.log(`Sending push notification to user ${userId}`);
    // Get the device tokens for the user

    const tokens = await this.deviceService.getTokens(userId);

    if (!tokens.length) {
      this.logger.log(`No device tokens found for user ${userId} but notification created in the database`);
      return;
    }
    // Send the notification
    return this.firebaseService.sendNotification(tokens, title, body);
  }

  private async sendSMS(to: string, body: string) {
    this.logger.log(`Sending SMS to ${to}`);
    await this.twilioService.sendSMS(to, body);
  }

  private async sendEmail({ to, subject, data, template }: SendEmailRequestDto) {
    this.logger.log(`Sending email to ${to}`);
    await this.mailerService.sendMail({
      to,
      subject,
      template,
      context: { ...data },
    });
    this.logger.log(`Email sent to ${to}`);
  }

  private getTemplateFromNotification(notification: Notification) {
    switch (notification.scope) {
      case NotificationScope.OTP:
        return 'otp';
      case NotificationScope.USER_INVITED:
        return 'user-invite';
      default:
        return 'otp';
    }
  }

  @OnEvent(EventType.NOTIFICATION_CREATED)
  handleNotificationCreatedEvent(notification: Notification, data?: any) {
    this.logger.log(
      `Handling ${EventType.NOTIFICATION_CREATED} event for notification ${notification.id} and type ${notification.channel}`,
    );
    switch (notification.channel) {
      case NotificationChannel.SMS:
        return this.sendSMS(notification.recipient!, notification.message);
      case NotificationChannel.PUSH:
        return this.sendPushNotification(notification.userId, notification.title, notification.message);
      case NotificationChannel.EMAIL:
        return this.sendEmail({
          to: notification.recipient!,
          subject: notification.title,
          template: this.getTemplateFromNotification(notification),
          data,
        });
    }
  }
}
