import { Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { FindOptionsWhere, Repository } from 'typeorm';
import { CustomerFiltersRequestDto } from '../dtos/request';
import { Customer } from '../entities';

@Injectable()
export class CustomerRepository {
  constructor(@InjectRepository(Customer) private readonly customerRepository: Repository<Customer>) {}

  updateCustomer(id: string, data: Partial<Customer>) {
    return this.customerRepository.update(id, data);
  }

  findOne(where: FindOptionsWhere<Customer>) {
    return this.customerRepository.findOne({
      where,
      relations: ['profilePicture', 'user', 'civilIdFront', 'civilIdBack'],
    });
  }

  createCustomer(userId: string, body: Partial<Customer>, isGuardian: boolean = false) {
    return this.customerRepository.save(
      this.customerRepository.create({
        id: userId,
        userId,
        isGuardian,
        isJunior: !isGuardian,
        firstName: body.firstName,
        lastName: body.lastName,
        dateOfBirth: body.dateOfBirth,
        gender: body.gender,
        countryOfResidence: body.countryOfResidence,
        nationalId: body.nationalId,
        nationalIdExpiry: body.nationalIdExpiry,
        sourceOfIncome: body.sourceOfIncome,
        profession: body.profession,
        professionType: body.professionType,
        isPep: body.isPep,

        civilIdFrontId: body.civilIdFrontId,
        civilIdBackId: body.civilIdBackId,
      }),
    );
  }

  findCustomers(filters: CustomerFiltersRequestDto) {
    const query = this.customerRepository.createQueryBuilder('customer');
    query.leftJoinAndSelect('customer.profilePicture', 'profilePicture');
    query.leftJoinAndSelect('customer.user', 'user');

    if (filters.name) {
      const nameParts = filters.name.trim().split(/\s+/);
      console.log(nameParts);
      nameParts.length > 1
        ? query.andWhere('customer.firstName LIKE :firstName AND customer.lastName LIKE :lastName', {
            firstName: `%${nameParts[0]}%`,
            lastName: `%${nameParts[1]}%`,
          })
        : query.andWhere('customer.firstName LIKE :name OR customer.lastName LIKE :name', {
            name: `%${filters.name.trim()}%`,
          });
    }

    if (filters.kycStatus) {
      query.andWhere('customer.kycStatus = :kycStatus', { kycStatus: filters.kycStatus });
    }

    query.orderBy('customer.createdAt', 'DESC');
    query.take(filters.size);
    query.skip((filters.page - 1) * filters.size);

    return query.getManyAndCount();
  }

  findCustomerByCivilId(civilIdFrontId: string, civilIdBackId: string) {
    return this.customerRepository.findOne({
      where: [{ civilIdFrontId, civilIdBackId }, { civilIdFrontId }, { civilIdBackId }],
    });
  }
}
