import { Body, Controller, Post, UploadedFile, UseGuards, UseInterceptors } from '@nestjs/common';
import { FileInterceptor } from '@nestjs/platform-express';
import { ApiBearerAuth, ApiBody, ApiConsumes, ApiTags } from '@nestjs/swagger';
import { memoryStorage } from 'multer';
import { IJwtPayload } from '~/auth/interfaces';
import { AuthenticatedUser } from '~/common/decorators';
import { AccessTokenGuard } from '~/common/guards';
import { ApiLangRequestHeader } from '~/core/decorators';
import { ResponseFactory } from '~/core/utils';
import { UploadDocumentRequestDto } from '../dtos/request';
import { DocumentMetaResponseDto } from '../dtos/response';
import { DocumentType } from '../enums';
import { DocumentService } from '../services';
@Controller('document')
@ApiTags('Document')
@ApiBearerAuth()
@UseGuards(AccessTokenGuard)
@ApiLangRequestHeader()
export class DocumentController {
  constructor(private readonly documentService: DocumentService) {}

  @Post()
  @ApiConsumes('multipart/form-data')
  @ApiBody({
    schema: {
      type: 'object',
      properties: {
        document: {
          type: 'string',
          format: 'binary',
        },
        documentType: {
          type: 'string',
          enum: Object.values(DocumentType).filter(
            (value) => ![DocumentType.DEFAULT_AVATAR, DocumentType.DEFAULT_TASKS_LOGO].includes(value),
          ),
        },
      },
      required: ['document', 'documentType'],
    },
  })
  @UseInterceptors(FileInterceptor('document', { storage: memoryStorage() }))
  async createDocument(
    @UploadedFile() file: Express.Multer.File,
    @Body() uploadedDocumentRequest: UploadDocumentRequestDto,
    @AuthenticatedUser() user: IJwtPayload,
  ) {
    const document = await this.documentService.createDocument(file, uploadedDocumentRequest, user.sub);

    return ResponseFactory.data(new DocumentMetaResponseDto(document));
  }
}
