import { BadRequestException, Injectable, Logger } from '@nestjs/common';
import { OciService } from '~/document/services';
import { JuniorService } from '~/junior/services';
import { CreateMoneyRequestRequestDto, MoneyRequestsFiltersRequestDto } from '../dtos/request';
import { MoneyRequest } from '../entities';
import { MoneyRequestFrequency, MoneyRequestStatus } from '../enums';
import { MoneyRequestsRepository } from '../repositories';

@Injectable()
export class MoneyRequestsService {
  private readonly logger = new Logger(MoneyRequestsService.name);
  constructor(
    private readonly moneyRequestsRepository: MoneyRequestsRepository,
    private readonly juniorService: JuniorService,
    private readonly ociService: OciService,
  ) {}

  async createMoneyRequest(userId: string, body: CreateMoneyRequestRequestDto) {
    this.logger.log(`Creating money request for junior ${userId}`);
    if (body.frequency === MoneyRequestFrequency.ONE_TIME) {
      this.logger.log(`Setting end date to null for one time money request`);
      delete body.endDate;
      delete body.startDate;
    }

    if (body.startDate && new Date(body.startDate) < new Date()) {
      this.logger.error(`Start date ${body.startDate} is in the past`);
      throw new BadRequestException('MONEY_REQUEST.START_DATE_IN_THE_PAST');
    }

    if (body.endDate && new Date(body.endDate) < new Date()) {
      this.logger.error(`End date ${body.endDate} is in the past`);
      throw new BadRequestException('MONEY_REQUEST.END_DATE_IN_THE_PAST');
    }

    if (body.endDate && body.startDate && new Date(body.endDate) < new Date(body.startDate)) {
      this.logger.error(`End date ${body.endDate} is before start date ${body.startDate}`);
      throw new BadRequestException('MONEY_REQUEST.END_DATE_BEFORE_START_DATE');
    }

    const junior = await this.juniorService.findJuniorById(userId, true);

    const moneyRequest = await this.moneyRequestsRepository.createMoneyRequest(junior.id, junior.guardianId, body);
    this.logger.log(`Money request ${moneyRequest.id} created successfully`);
    return this.findMoneyRequestById(moneyRequest.id);
  }

  async findMoneyRequestById(moneyRequestId: string, reviewerId?: string) {
    this.logger.log(`Finding money request ${moneyRequestId} ${reviewerId ? `by reviewer ${reviewerId}` : ''}`);
    const moneyRequest = await this.moneyRequestsRepository.findMoneyRequestById(moneyRequestId, reviewerId);

    if (!moneyRequest) {
      this.logger.error(`Money request ${moneyRequestId} not found ${reviewerId ? `for reviewer ${reviewerId}` : ''}`);
      throw new BadRequestException('MONEY_REQUEST.NOT_FOUND');
    }

    await this.prepareMoneyRequestDocument([moneyRequest]);

    this.logger.log(`Money request ${moneyRequestId} found successfully`);
    return moneyRequest;
  }

  async findMoneyRequests(
    ReviewerId: string,
    filters: MoneyRequestsFiltersRequestDto,
  ): Promise<[MoneyRequest[], number]> {
    this.logger.log(`Finding money requests for reviewer ${ReviewerId}`);
    const [moneyRequests, itemCount] = await this.moneyRequestsRepository.findMoneyRequests(ReviewerId, filters);
    await this.prepareMoneyRequestDocument(moneyRequests);

    this.logger.log(`Returning money requests for reviewer ${ReviewerId}`);
    return [moneyRequests, itemCount];
  }

  async approveMoneyRequest(moneyRequestId: string, reviewerId: string) {
    this.logger.log(`Approving money request ${moneyRequestId} by reviewer ${reviewerId}`);
    await this.validateMoneyRequestForReview(moneyRequestId, reviewerId);
    await this.moneyRequestsRepository.updateMoneyRequestStatus(
      moneyRequestId,
      reviewerId,
      MoneyRequestStatus.APPROVED,
    );

    //@TODO send notification and update junior balance
  }

  async rejectMoneyRequest(moneyRequestId: string, reviewerId: string) {
    this.logger.log(`Rejecting money request ${moneyRequestId} by reviewer ${reviewerId}`);
    await this.validateMoneyRequestForReview(moneyRequestId, reviewerId);

    await this.moneyRequestsRepository.updateMoneyRequestStatus(
      moneyRequestId,
      reviewerId,
      MoneyRequestStatus.REJECTED,
    );

    //@TODO send notification
  }

  findMoneyRequestsChunks(chunkSize: number) {
    this.logger.log(`Finding money requests chunks`);
    return this.moneyRequestsRepository.findMoneyRequestsChunks(chunkSize);
  }

  private async prepareMoneyRequestDocument(moneyRequests: MoneyRequest[]) {
    this.logger.log(`Preparing document for money requests`);
    await Promise.all(
      moneyRequests.map(async (moneyRequest) => {
        const profilePicture = moneyRequest.requester.customer.profilePicture;

        if (profilePicture) {
          profilePicture.url = await this.ociService.generatePreSignedUrl(profilePicture);
        }
      }),
    );
  }

  private async validateMoneyRequestForReview(moneyRequestId: string, reviewerId: string) {
    this.logger.log(`Validating money request ${moneyRequestId} for reviewer ${reviewerId}`);
    const moneyRequest = await this.moneyRequestsRepository.findMoneyRequestById(moneyRequestId, reviewerId);

    if (!moneyRequest) {
      this.logger.error(`Money request ${moneyRequestId} not found`);
      throw new BadRequestException('MONEY_REQUEST.NOT_FOUND');
    }

    if (moneyRequest.endDate && new Date(moneyRequest.endDate) < new Date()) {
      this.logger.error(`Money request ${moneyRequestId} has already ended`);
      throw new BadRequestException('MONEY_REQUEST.ENDED');
    }

    if (moneyRequest.status !== MoneyRequestStatus.PENDING) {
      this.logger.error(`Money request ${moneyRequestId} already reviewed`);
      throw new BadRequestException('MONEY_REQUEST.ALREADY_REVIEWED');
    }
  }
}
