import { BadRequestException, Injectable, Logger } from '@nestjs/common';
import moment from 'moment';
import { PageOptionsRequestDto } from '~/core/dtos';
import { OciService } from '~/document/services';
import { CreateGoalRequestDto, FundGoalRequestDto } from '../dtos/request';
import { SavingGoal } from '../entities';
import { IGoalStats } from '../interfaces';
import { SavingGoalsRepository } from '../repositories';
import { CategoryService } from './category.service';
@Injectable()
export class SavingGoalsService {
  private readonly logger = new Logger(SavingGoalsService.name);
  constructor(
    private readonly savingGoalsRepository: SavingGoalsRepository,
    private readonly categoryService: CategoryService,
    private readonly ociService: OciService,
  ) {}

  async createGoal(juniorId: string, body: CreateGoalRequestDto) {
    this.logger.log(`Creating goal for junior ${juniorId}`);
    if (moment(body.dueDate).isBefore(moment())) {
      this.logger.error(`Due date must be in the future`);
      throw new BadRequestException('GOAL.DUE_DATE_MUST_BE_IN_THE_FUTURE');
    }

    const categories = await this.categoryService.findCategoryByIds(body.categoryIds, juniorId);

    const createdGoal = await this.savingGoalsRepository.createGoal(juniorId, body, categories);

    this.logger.log(`Goal ${createdGoal.id} created successfully`);

    return this.findGoalById(juniorId, createdGoal.id);
  }

  async findGoals(juniorId: string, pageOptions: PageOptionsRequestDto): Promise<[SavingGoal[], number]> {
    this.logger.log(`Finding goals for junior ${juniorId}`);
    const [goals, itemCount] = await this.savingGoalsRepository.findGoals(juniorId, pageOptions);
    await this.prepareGoalsImages(goals);

    this.logger.log(`Returning goals for junior ${juniorId}`);
    return [goals, itemCount];
  }

  async findGoalById(juniorId: string, goalId: string) {
    this.logger.log(`Finding goal ${goalId} for junior ${juniorId}`);
    const goal = await this.savingGoalsRepository.findGoalById(juniorId, goalId);

    if (!goal) {
      this.logger.error(`Goal ${goalId} not found`);
      throw new BadRequestException('GOAL.NOT_FOUND');
    }

    await this.prepareGoalsImages([goal]);

    this.logger.log(`Goal ${goalId} found successfully`);
    return goal;
  }

  async fundGoal(juniorId: string, goalId: string, body: FundGoalRequestDto) {
    this.logger.log(`Funding goal ${goalId} for junior ${juniorId}`);
    const goal = await this.savingGoalsRepository.findGoalById(juniorId, goalId);

    if (!goal) {
      this.logger.error(`Goal ${goalId} not found`);
      throw new BadRequestException('GOAL.NOT_FOUND');
    }

    if (goal.currentAmount + body.fundAmount > goal.targetAmount) {
      this.logger.error(
        `Funding amount exceeds total amount , currentAmount: ${goal.currentAmount}, fundAmount: ${body.fundAmount}, targetAmount: ${goal.targetAmount}`,
      );
      throw new BadRequestException('GOAL.FUND_EXCEEDS_TOTAL_AMOUNT');
    }

    await this.savingGoalsRepository.fundGoal(juniorId, goalId, body.fundAmount);
    this.logger.log(`Goal ${goalId} funded successfully with amount ${body.fundAmount}`);
  }

  async getStats(juniorId: string): Promise<IGoalStats> {
    this.logger.log(`Getting stats for junior ${juniorId}`);
    const result = await this.savingGoalsRepository.getStats(juniorId);
    return {
      totalTarget: result?.totalTarget,
      totalSaved: result?.totalSaved,
    };
  }

  private async prepareGoalsImages(goals: SavingGoal[]) {
    this.logger.log(`Preparing images for goals`);
    await Promise.all(
      goals.map(async (goal) => {
        if (goal.imageId) {
          goal.image.url = await this.ociService.generatePreSignedUrl(goal.image);
        }
      }),
    );
  }
}
